
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFExtFile.cacheable import Cacheable
from Products.ExtFile import ExtFile
from Products.ExtFile import ExtImage

from Products.CMFExtFile.config import manage_titles
from Products.CMFExtFile.config import preview_mode
from Products.CMFExtFile.config import preview_maxx
from Products.CMFExtFile.config import preview_maxy
from Products.CMFExtFile.config import preview_ratio

try:
    from zope.interface import implements
except ImportError:
    implements = None

try:
    from Products.CMFExtFile.interfaces import ICMFExtImage
except ImportError:
    ICMFExtImage = None

try:
    from Products.GenericSetup.interfaces import IDAVAware
except ImportError:
    IDAVAware = None

try:
    from zope.component.factory import Factory
except ImportError:
    Factory = None


factory_type_information = (
  { 'id'             : 'ExtImage'
  , 'title'          : 'ExtImage'
  , 'meta_type'      : 'Portal ExtImage'
  , 'description'    : """\
ExtImage objects can contain image files.
"""
  , 'icon'           : 'image_icon.gif'
  , 'product'        : 'CMFExtFile'
  , 'factory'        : 'addExtImage'
  , 'immediate_view' : 'metadata_edit_form'
  , 'aliases'        : {'(Default)': 'index_html',
                        'view': 'extimage_view',
                        'edit': 'image_edit_form',
                        'metadata': 'metadata_edit_form'}
  , 'actions'        : ( { 'id'            : 'view'
                         , 'name'          : 'View'
                         , 'title'         : 'View'
                         , 'action': 'string:${object_url}/view'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'download'
                         , 'name'          : 'Download'
                         , 'title'         : 'Download'
                         , 'action': 'string:${object/static_url}'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'edit'
                         , 'name'          : 'Edit'
                         , 'title'         : 'Edit'
                         , 'action': 'string:${object_url}/edit'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       , { 'id'            : 'metadata'
                         , 'name'          : 'Metadata'
                         , 'title'         : 'Metadata'
                         , 'action': 'string:${object_url}/metadata'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       )
  }
,
)


def addExtImage( self
            , id=''
            , title=''
            , file=''
            , content_type=''
            , precondition=''
            , subject=()
            , description=''
            , contributors=()
            , effective_date=None
            , expiration_date=None
            , format=None
            , language=''
            , rights=''
            ):
    """
        Add an ExtImage
    """

    if not id and getattr(file, 'filename', None) is not None:
        # Generate id from filename and make sure it has no 'bad' chars
        id = file.filename
        id = id[max(id.rfind('/'),
                    id.rfind('\\'),
                    id.rfind(':'))+1:]
        title = title or id
        id = ExtFile.normalize_id(id)

    self=self.this()

    # Instantiate the object and set its description.
    iobj = CMFExtImage( id, title, '', content_type, precondition, subject
                , description, contributors, effective_date, expiration_date
                , format, language, rights
                )

    # Add the Image instance to self
    self._setObject(id, iobj)

    # 'Upload' the image.  This is done now rather than in the
    # constructor because it's faster (see File.py.)
    if file != '':
        self._getOb(id).manage_upload(file, content_type)
        self._getOb(id).updatePreview()

    return id


class CMFExtImage( PortalContent
           , Cacheable
           , ExtImage.ExtImage
           , DefaultDublinCoreImpl
           ):
    """
        A Portal-managed ExtImage
    """

    if implements is not None:
        implements(*filter(None, (ICMFExtImage, IDAVAware)))

    if hasattr(DefaultDublinCoreImpl, '__implements__'):
        __implements__ = ( PortalContent.__implements__
                         , DefaultDublinCoreImpl.__implements__
                         )

    meta_type='Portal ExtImage'
    effective_date = expiration_date = None
    _isDiscussable = 1
    precondition = ''
    index_html = ExtImage.ExtImage.index_html

    manage_options = (
        ExtImage.ExtImage.manage_options[:3] +
        PortalContent.manage_options[:1] +
        CMFCatalogAware.manage_options +
        ExtImage.ExtImage.manage_options[3:]
    )

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , file=''
                , content_type=''
                , precondition=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format=None
                , language='en-US'
                , rights=''
                ):
        ExtImage.ExtImage.__init__( self, id, title, description
                               , permission_check=0
                               , redirect_default_view=0 )

        # If no file format has been passed in, rely on what OFS.Image.File
        # detected.
        if format is None:
            format = self.content_type

        DefaultDublinCoreImpl.__init__( self, title, subject, description
                               , contributors, effective_date, expiration_date
                               , format, language, rights )

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """
            SearchableText is used for full text searches of a portal.
            Returns Title and Description of the image.
        """
        return "%s %s" % (self.Title(), self.Description())

    security.declarePrivate('_isNotEmpty')
    def _isNotEmpty(self, file):
        """ Do various checks on 'file' to try to determine non emptiness. """
        if not file:
            return 0                    # Catches None, Missing.Value, ''
        elif file and (type(file) is type('')):
            return 1
        elif getattr(file, 'filename', None):
            return 1
        elif not getattr(file, 'read', None):
            return 0
        else:
            file.seek(0,2)              # 0 bytes back from end of file
            t = file.tell()             # Report the location
            file.seek(0)                # and return pointer back to 0
            if t: return 1
            else: return 0

    security.declarePrivate('_edit')
    def _edit(self, precondition='', file=''):
        """ Update image. """
        if precondition: self.precondition = precondition
        elif self.precondition: del self.precondition

        if self._isNotEmpty(file):
            self.manage_upload(file)
            self.updatePreview()

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, precondition='', file=''):
        """ Update and reindex. """
        self._edit( precondition, file )
        self.reindexObject()

    security.declareProtected(View, 'Format')
    def Format(self):
        """ Dublin Core element - resource format """
        return self.content_type

    security.declareProtected(ModifyPortalContent, 'setFormat')
    def setFormat(self, format):
        """ Dublin Core element - resource format """
        self.manage_changeProperties(content_type=format)

    security.declareProtected(ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP (and FTP) PUT requests """
        self.has_preview = False
        ExtImage.ExtImage.PUT( self, REQUEST, RESPONSE )
        self.updatePreview()
        if manage_titles(self) and not self.Title():
            self.setTitle(self.getId())
        self.reindexObject()

    security.declareProtected(ModifyPortalContent, 'createPreview')
    def createPreview(self, maxx=None, maxy=None, ratio=None):
        """ Create preview image. """
        if maxx is None:
            maxx = preview_maxx(self)
        if maxy is None:
            maxy = preview_maxy(self)
        if ratio is None:
            ratio = preview_ratio(self)
        return self.manage_create_prev(maxx, maxy, ratio)

    security.declareProtected(ModifyPortalContent, 'deletePreview')
    def deletePreview(self):
        """ Delete preview image. """
        self.manage_del_prev()

    security.declareProtected(ModifyPortalContent, 'updatePreview')
    def updatePreview(self):
        """ Create or delete preview, depending on tool settings. """
        if preview_mode(self) == ExtImage.GENERATE:
            self.createPreview()
        else:
            self.deletePreview()


InitializeClass(CMFExtImage)

if Factory is not None:
    ExtImageFactory = Factory(CMFExtImage)

