
from Acquisition import aq_base

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.TypesTool import FactoryTypeInformation
from Products.CMFCore.DirectoryView import addDirectoryViews

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import AccessContentsInformation

from Products.CMFExtFile import CMFExtFile
from Products.CMFExtFile import CMFExtImage
from Products.CMFExtFile import cmfextfile_globals

try:
    from Products.CMFCore.ActionInformation import Action
except ImportError:
    # CMF < 2.0
    Action = None


def install(self, reinstall=0):
    '''Installs the CMFExtFile portal types and tool.'''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    for fti in CMFExtImage.factory_type_information + CMFExtFile.factory_type_information:
        portal_type = fti['id']
        if getattr(aq_base(types), portal_type, None) is not None:
            types._delObject(portal_type)
        cfm = FactoryTypeInformation(**fti)
        types._setObject(portal_type, cfm)

    skins = getToolByName(self, 'portal_skins', None)
    if skins is None:
        return

    if getattr(aq_base(skins), 'extfile', None) is None:
        addDirectoryViews(skins, 'skins', cmfextfile_globals)

    selections = skins.getSkinSelections()
    for skin_id in selections:
        path = skins.getSkinPath(skin_id)
        path = [x.strip() for x in path.split(',')]
        path = [x for x in path if x]
        if 'extfile' not in path:
            if 'custom' in path:
                path.insert(path.index('custom')+1, 'extfile')
            else:
                path.append('extfile')
            skins.addSkinSelection(skin_id, ','.join(path))

    if getattr(aq_base(self), 'extfile_properties', None) is None:
        factory = self.manage_addProduct['CMFExtFile']
        factory.manage_addTool('CMF ExtFile Tool')

    if not reinstall:
        replace_file_and_image(self)
        add_thumbnails_action(self)

    return 'Successfully installed CMFExtFile.'


def uninstall(self, reinstall=0):
    '''Uninstalls the CMFExtFile portal types and tool.'''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    for portal_type in ('ExtImage', 'ExtFile'):
        if getattr(aq_base(types), portal_type, None) is not None:
            types._delObject(portal_type)

    skins = getToolByName(self, 'portal_skins', None)
    if skins is None:
        return

    if getattr(aq_base(skins), 'extfile', None) is not None:
        skins._delObject('extfile')

    selections = skins.getSkinSelections()
    for skin_id in selections:
        path = skins.getSkinPath(skin_id)
        path = [x.strip() for x in path.split(',')]
        path = [x for x in path if x]
        if 'extfile' in path:
            path.remove('extfile')
            skins.addSkinSelection(skin_id, ','.join(path))

    if not reinstall:
        if getattr(aq_base(self), 'extfile_properties', None) is not None:
            self._delObject('extfile_properties')

        restore_file_and_image(self)
        remove_thumbnails_action(self)

    return 'Successfully uninstalled CMFExtFile.'


def replace_file_and_image(self):
    '''Replaces File and Image portal types with
       ExtFile and ExtImage respectively.
    '''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    if getattr(aq_base(types), 'File', None) is not None:
        types.File.manage_changeProperties(global_allow=False)

    if getattr(aq_base(types), 'Image', None) is not None:
        types.Image.manage_changeProperties(global_allow=False)

    if getattr(aq_base(types), 'ExtFile', None) is not None:
        types.ExtFile.manage_changeProperties(title='File',
                description='File objects can contain arbitrary downloadable files.')

    if getattr(aq_base(types), 'ExtImage', None) is not None:
        types.ExtImage.manage_changeProperties(title='Image',
                description='Image objects can contain image files.')

    registry = getToolByName(self, 'content_type_registry', None)
    if registry is None:
        return

    predicates = registry.predicate_ids
    if 'file' in predicates:
        registry.assignTypeName('file', 'ExtFile')
    if 'image' in predicates:
        registry.assignTypeName('image', 'ExtImage')


def restore_file_and_image(self):
    '''Restores File and Image portal types.
    '''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    if getattr(aq_base(types), 'File', None) is not None:
        types.File.manage_changeProperties(global_allow=True)

    if getattr(aq_base(types), 'Image', None) is not None:
        types.Image.manage_changeProperties(global_allow=True)

    if getattr(aq_base(types), 'ExtFile', None) is not None:
        types.ExtFile.manage_changeProperties(title='ExtFile',
                description='ExtFile objects can contain arbitrary downloadable files.')

    if getattr(aq_base(types), 'ExtImage', None) is not None:
        types.ExtImage.manage_changeProperties(title='ExtImage',
                description='ExtImage objects can contain image files.')

    registry = getToolByName(self, 'content_type_registry', None)
    if registry is None:
        return

    predicates = registry.predicate_ids
    if 'file' in predicates:
        registry.assignTypeName('file', 'File')
    if 'image' in predicates:
        registry.assignTypeName('image', 'Image')


def add_thumbnails_action(self):
    '''Adds a 'Thumbnails' action to Folders.'''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    actions = getToolByName(self, 'portal_actions', None)
    if actions is None:
        return

    if getattr(aq_base(types), 'Folder', None) is not None:
        if types.Folder.getActionObject('object/thumbnails') is None:
            types.Folder.addAction( 'thumbnails'
                                  , name='Thumbnails'
                                  , action='string:${object_url}/thumbnail_view'
                                  , condition="python: object.objectIds(['Portal ExtImage'])"
                                  , permission=(AccessContentsInformation,)
                                  , category='object'
                                  , visible=True
                                  )

    if Action is None:
        if actions.getActionObject('folder/thumbnails') is None:
            actions.addAction( 'thumbnails'
                             , name='Thumbnails'
                             , action='string:${folder_url}/thumbnail_view'
                             , condition="python: folder is not object and folder.objectIds(['Portal ExtImage'])"
                             , permission=(AccessContentsInformation,)
                             , category='folder'
                             , visible=True
                             )
    else:
        if 'folder' in actions.objectIds():
            if 'thumbnails' not in actions.folder.objectIds():
                thumbnails = Action( 'thumbnails'
                                   , title='Thumbnails'
                                   , description=''
                                   , i18n_domain=''
                                   , url_expr='string:${object_url}/thumbnail_view'
                                   , icon_expr=''
                                   , available_expr="python: object.objectIds(['Portal ExtImage'])"
                                   , permissions=(AccessContentsInformation,)
                                   , visible=True
                                   )
                actions.folder._setObject('thumbnails', thumbnails)


def remove_thumbnails_action(self):
    '''Removes the 'Thumbnails action from Folders.'''
    types = getToolByName(self, 'portal_types', None)
    if types is None:
        return

    actions = getToolByName(self, 'portal_actions', None)
    if actions is None:
        return

    if getattr(aq_base(types), 'Folder', None) is not None:
        if types.Folder.getActionObject('object/thumbnails') is not None:
            new_actions = []
            for action in types.Folder._actions:
                if not (action.getId() == 'thumbnails' and action.getCategory() == 'object'):
                    new_actions.append(action)
            types.Folder._actions = tuple(new_actions)

    if actions.getActionObject('folder/thumbnails') is not None:
        new_actions = []
        for action in actions._actions:
            if not (action.getId() == 'thumbnails' and action.getCategory() == 'folder'):
                new_actions.append(action)
        actions._actions = tuple(new_actions)

    if 'folder' in actions.objectIds():
        if 'thumbnails' in actions.folder.objectIds():
            actions.folder._delObject('thumbnails')

