
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from ExtensionClass import Base
from Acquisition import aq_base
from Acquisition import aq_acquire
from OFS import Cache

from OFS.Cache import ChangeCacheSettingsPermission
from Products.CMFCore.permissions import ViewManagementScreens

from Products.CMFExtFile.config import cache_manager


class Cacheable(Base):
    """Mix-in allowing objects to use the cache manager id
       defined in the extfile_properties tool.
    """

    security = ClassSecurityInfo()

    security.declareProtected(ViewManagementScreens, 'ZCacheable_getManagerId')
    def ZCacheable_getManagerId(self):
        '''Returns the id of the current ZCacheManager.'''
        manager_id = self._Cacheable__manager_id
        if manager_id is None:
            manager_id = cache_manager(self)
        if manager_id:
            return manager_id

    security.declareProtected(ChangeCacheSettingsPermission, 'ZCacheable_setManagerId')
    def ZCacheable_setManagerId(self, manager_id, REQUEST=None):
        '''Changes the manager_id for this object.'''
        self.ZCacheable_invalidate()
        if not manager_id or manager_id == cache_manager(self):
            # User requested disassociation
            # from the cache manager.
            manager_id = None
        else:
            manager_id = str(manager_id)
        self._Cacheable__manager_id = manager_id
        self._v_ZCacheable_cache = None
        if REQUEST is not None:
            return self.ZCacheable_manage(
                self, REQUEST, management_view='Cache',
                manage_tabs_message='Cache settings changed.')

    security.declarePrivate('ZCacheable_getManager')
    def ZCacheable_getManager(self):
        '''Returns the currently associated cache manager.'''
        manager_id = self.ZCacheable_getManagerId()
        if manager_id is None:
            return None
        try:
            return aq_acquire(
                self, manager_id, containment=1,
                filter=Cache.filterCacheManagers, extra=None, default=None)
        except AttributeError:
            return None

    security.declarePrivate('ZCacheable_getCache')
    def ZCacheable_getCache(self):
        '''Gets the cache associated with this object.
        '''
        if self.ZCacheable_getManagerId() is None:
            return None
        c = self._v_ZCacheable_cache
        if c is not None:
            # We have a volatile reference to the cache.
            if self._v_ZCacheable_manager_timestamp == Cache.manager_timestamp:
                return aq_base(c)
        manager = self.ZCacheable_getManager()
        if manager is not None:
            c = aq_base(manager.ZCacheManager_getCache())
        else:
            return None
        # Set a volatile reference to the cache then return it.
        self._v_ZCacheable_cache = c
        self._v_ZCacheable_manager_timestamp = Cache.manager_timestamp
        return c


InitializeClass(Cacheable)

