#
# Event tests
#

from Products.CMFExtFile import testing
from Testing.ZopeTestCase import transaction

gifImage = testing.gifImage

try:
    from OFS import subscribers
except ImportError:
    subscribers = None


class TestEvents(testing.CMFExtFileTestCase):

    def afterSetUp(self):
        testing.CMFExtFileTestCase.afterSetUp(self)
        self.catalog = self.portal.portal_catalog

    def testAddExtFile(self):
        self.folder.invokeFactory('ExtFile', id='file', file=open(gifImage, 'rb'))
        self.assertEqual(self.folder.file.Title(), 'file')
        self.assertEqual(len(self.catalog(Title='file')), 1)

    def testAddExtImage(self):
        self.folder.invokeFactory('ExtImage', id='image', file=open(gifImage, 'rb'))
        self.assertEqual(self.folder.image.Title(), 'image')
        self.assertEqual(len(self.catalog(Title='image')), 1)

    def testRenameExtFile(self):
        self.folder.invokeFactory('ExtFile', id='file', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('file', 'file2')
        self.assertEqual(self.folder.file2.Title(), 'file2')
        self.assertEqual(len(self.catalog(Title='file2')), 1)

    def testRenameExtImage(self):
        self.folder.invokeFactory('ExtImage', id='image', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('image', 'image2')
        self.assertEqual(self.folder.image2.Title(), 'image2')
        self.assertEqual(len(self.catalog(Title='image2')), 1)

    def testRenameCopyOfExtFile(self):
        self.folder.invokeFactory('ExtFile', id='copy_of_file', title='file', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('copy_of_file', 'file2')
        self.assertEqual(self.folder.file2.Title(), 'file2')
        self.assertEqual(len(self.catalog(Title='file2')), 1)

    def testRenameCopyOfExtImage(self):
        self.folder.invokeFactory('ExtImage', id='copy_of_image', title='image', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('copy_of_image', 'image2')
        self.assertEqual(self.folder.image2.Title(), 'image2')
        self.assertEqual(len(self.catalog(Title='image2')), 1)

    def testRenameCopyOfExtFileTitleFixup(self):
        self.folder.invokeFactory('ExtFile', id='copy_of_file', title='copy_of_file', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('copy_of_file', 'file2')
        self.assertEqual(self.folder.file2.Title(), 'file2')
        self.assertEqual(len(self.catalog(Title='file2')), 1)

    def testRenameCopyOfExtImageTitleFixup(self):
        self.folder.invokeFactory('ExtImage', id='copy_of_image', title='copy_of_image', file=open(gifImage, 'rb'))
        transaction.savepoint(True)
        self.folder.manage_renameObject('copy_of_image', 'image2')
        self.assertEqual(self.folder.image2.Title(), 'image2')
        self.assertEqual(len(self.catalog(Title='image2')), 1)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    if subscribers is not None:
        suite.addTest(makeSuite(TestEvents))
    return suite

