#
# Tests the interfaces
#

from Products.CMFExtFile import testing

from zope.interface.verify import verifyObject
from zope.interface.verify import verifyClass
from zope.interface.exceptions import DoesNotImplement

from Products.ExtFile.interfaces import IExtFile
from Products.ExtFile.interfaces import IExtImage
from Products.CMFExtFile.interfaces import ICMFExtFile
from Products.CMFExtFile.interfaces import ICMFExtImage
from Products.CMFExtFile.CMFExtFile import CMFExtFile
from Products.CMFExtFile.CMFExtImage import CMFExtImage

try:
    from Products.GenericSetup.interfaces import IDAVAware
except ImportError:
    IDAVAware = None


class TestInterfaces(testing.CMFExtFileTestCase):

    def afterSetUp(self):
        super(TestInterfaces, self).afterSetUp()
        self.folder.invokeFactory('ExtFile', id='file')
        self.folder.invokeFactory('ExtImage', id='image')
        self.file = self.folder.file
        self.image = self.folder.image

    def testIExtFile(self):
        self.failUnless(verifyObject(IExtFile, self.file))
        self.failUnless(verifyClass(IExtFile, CMFExtFile))
        self.failUnless(verifyObject(IExtFile, self.image))
        self.failUnless(verifyClass(IExtFile, CMFExtImage))

    def testIExtImage(self):
        self.failUnless(verifyObject(IExtImage, self.image))
        self.failUnless(verifyClass(IExtImage, CMFExtImage))

    def testICMFExtFile(self):
        self.failUnless(verifyObject(ICMFExtFile, self.file))
        self.failUnless(verifyClass(ICMFExtFile, CMFExtFile))
        self.failUnless(verifyObject(ICMFExtFile, self.image))
        self.failUnless(verifyClass(ICMFExtFile, CMFExtImage))

    def testICMFExtImage(self):
        self.failUnless(verifyObject(ICMFExtImage, self.image))
        self.failUnless(verifyClass(ICMFExtImage, CMFExtImage))

    if IDAVAware is not None:

        def testIDAVAware(self):
            self.failUnless(verifyObject(IDAVAware, self.file))
            self.failUnless(verifyClass(IDAVAware, CMFExtFile))
            self.failUnless(verifyObject(IDAVAware, self.image))
            self.failUnless(verifyClass(IDAVAware, CMFExtImage))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInterfaces))
    return suite

