#
# CMFExtFile setup tests
#

from Products.CMFExtFile import testing
from Products.CMFExtFile.Extensions.Install import uninstall
from Products.CMFTestCase.setup import CMF16, CMF20, CMF21


class TestSetup(testing.CMFExtFileTestCase):

    def afterSetUp(self):
        testing.CMFExtFileTestCase.afterSetUp(self)
        self.types = self.portal.portal_types
        self.skins = self.portal.portal_skins
        self.props = self.portal.extfile_properties
        self.actions = self.portal.portal_actions
        self.registry = self.portal.content_type_registry

    def testTypes(self):
        ids = self.types.objectIds()
        self.failUnless('ExtFile' in ids)
        self.failUnless('ExtImage' in ids)

    def testTypeTitle(self):
        self.assertEqual(self.types.ExtFile.Title(), 'File')
        self.assertEqual(self.types.ExtImage.Title(), 'Image')

    def testSkins(self):
        ids = self.skins.objectIds()
        self.failUnless('extfile' in ids)

    def testSkinPaths(self):
        for skin_id in self.skins.getSkinSelections():
            path = self.skins.getSkinPath(skin_id)
            self.failUnless('extfile' in path)

    def testProperties(self):
        self.failUnless(self.props.hasProperty('cache_manager'))
        self.failUnless(self.props.hasProperty('create_previews'))
        self.failUnless(self.props.hasProperty('preview_maxx'))
        self.failUnless(self.props.hasProperty('preview_maxy'))
        self.failUnless(self.props.hasProperty('preview_ratio'))

    def testFileImageReplace(self):
        self.assertEqual(self.types.File.global_allow, False)
        self.assertEqual(self.types.Image.global_allow, False)

    def testContentTypeRegistry(self):
        self.assertEqual(self.registry.getTypeObjectName('file'), 'ExtFile')
        self.assertEqual(self.registry.getTypeObjectName('image'), 'ExtImage')

    def testThumbnailsAction(self):
        self.failIf(self.types.Folder.getActionObject('object/thumbnails') is None)
        if CMF20:
            self.failUnless('thumbnails' in self.actions.folder.objectIds())
        else:
            self.failIf(self.actions.getActionObject('folder/thumbnails') is None)


class TestUninstall(testing.CMFExtFileTestCase):

    def afterSetUp(self):
        testing.CMFExtFileTestCase.afterSetUp(self)
        self.types = self.portal.portal_types
        self.skins = self.portal.portal_skins
        self.actions = self.portal.portal_actions
        self.registry = self.portal.content_type_registry
        uninstall(self.portal)

    def testTypes(self):
        ids = self.types.objectIds()
        self.failIf('ExtFile' in ids)
        self.failIf('ExtImage' in ids)

    def testSkins(self):
        ids = self.skins.objectIds()
        self.failIf('extfile' in ids)

    def testSkinPaths(self):
        for skin_id in self.skins.getSkinSelections():
            path = self.skins.getSkinPath(skin_id)
            self.failIf('extfile' in path)

    def testProperties(self):
        ids = self.portal.objectIds()
        self.failIf('extfile_properties' in ids)

    def testFileImageRestore(self):
        self.assertEqual(self.types.File.global_allow, True)
        self.assertEqual(self.types.Image.global_allow, True)

    def testContentTypeRegistry(self):
        self.assertEqual(self.registry.getTypeObjectName('file'), 'File')
        self.assertEqual(self.registry.getTypeObjectName('image'), 'Image')

    def testThumbnailsAction(self):
        self.failUnless(self.types.Folder.getActionObject('object/thumbnails') is None)
        if CMF20:
            self.failIf('thumbnails' in self.actions.folder.objectIds())
        else:
            self.failUnless(self.actions.getActionObject('folder/thumbnails') is None)


class TestGenericSetup(TestSetup):

    def afterSetUp(self):
        testing.CMFExtFileTestCase.afterSetUp(self)
        self.types = self.portal.portal_types
        self.skins = self.portal.portal_skins
        self.props = self.portal.extfile_properties
        self.actions = self.portal.portal_actions
        self.registry = self.portal.content_type_registry

        uninstall(self.portal)

        setup = self.portal.portal_setup
        if CMF21:
            setup.runAllImportStepsFromProfile('profile-Products.CMFExtFile:default')
        else:
            saved = setup.getImportContextID()
            try:
                setup.setImportContext('profile-Products.CMFExtFile:default')
                setup.runAllImportSteps()
            finally:
                setup.setImportContext(saved)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSetup))
    suite.addTest(makeSuite(TestUninstall))
    if CMF16:
        suite.addTest(makeSuite(TestGenericSetup))
    return suite

